/*****************************************************************************************************************************************************************
*	File name 	: 	LCD_Display.C
*	Purpose		: 	Has functions to initialize and write data on 20 X 2 Character TWI display.
******************************************************************************************************************************************************************/

#include "main.h"


unsigned char buffer1[25] = {'S', 'N', 'D', 'R', ':', '0', '0', '.', '0', '0',
							 ' ', 'S', 'N', 'R', ':', '0', '0', '.', '0', '0'};
unsigned char buffer2[25] = {'E', 'N', 'O', 'B', ':', '0', '0', '.', '0', '0',
							 ' ', 'T', 'H', 'D', ':', '-', '0', '0', '.', '0'};
//	Note: though these arrays are defined as 25 bytes, first 20 bytes are used to store the characters

//	I2C Address of Character TWI display
unsigned char LCD_TWI_ADDR = 0x28;		

/*****************************************************************************************************************************************************************
*	Function name:  Init_Disaply
*	Description	:	This function calls a function to configure TWI peripheral of processor to which character display is connected and 
*					then displays default characters stored in buffer1[] on first line and characters stored in buffer2[] array on second line.
*	Arguments	:	None
*   Return value:   None.
*   Note		: 	In this code, this function is called at the start and buffer1[] & buffer2[] are not modified before calling Write_Disaply(). 
*					So, it displays the default values stored above. 
******************************************************************************************************************************************************************/

void Init_Disaply(void)
{
	Init_TWI0();
	Write_Disaply();
}


/*****************************************************************************************************************************************************************
*	Function name:  Write_Disaply
*	Description	:	This function displays the characters stored in buffer1[] array on first line and 
*					characters stored in buffer2[] array on second line. 
*	Arguments	:	None
*   Return value:   None.
*   Note		: 	buffer1[] and buffe2[] should have valid characters before calling this function.  
******************************************************************************************************************************************************************/

void Write_Disaply(void)
{
	Write_TWI_LCD_Command(LCD_TWI_ADDR, 0x45, 0x00, 3);		//	move display cursor at the start of first line	
	Write_TWI_LCD_Data(LCD_TWI_ADDR, buffer1, 20);			//	write the characters stored in buffer1[] array on first line
	Write_TWI_LCD_Command(LCD_TWI_ADDR, 0x45, 0x40, 3);		//	move display cursor at the start of second line	
	Write_TWI_LCD_Data(LCD_TWI_ADDR, buffer2, 20);			//	write the characters stored in buffer2[] array on first line
}


/*****************************************************************************************************************************************************************
*	Function name:  Display_ADC_para
*	Description	:	This function prepares the buffer1[] & buffer2[] arrays to display calculated ADC performance parameters on character display.
*					It converts floating point ADC performance parameter numbers into character stream and then calls Write_Disaply() to displat that stream.
*					SNDR & SNR are displayed on first line, while ENOB & THD are displayes on second line, as shown above (in the definition of buffer1 & buffer2).
*	Arguments	:	None. The function uses "ADC_para" structure (which is globally defined for storing calculated ADC performance parameters) directly.
*   Return value:   None.
*   Note		: 	A temporary stucture "ADC_para_temp" is used to convert the number in "ADC_para" into character stream.  
******************************************************************************************************************************************************************/

ADC_Perfomance_Metrices        ADC_para_temp;

void Display_ADC_para(void)
{
	ADC_para_temp = ADC_para; 

		buffer1[5] = (unsigned char) (ADC_para_temp.SNDR)/(10.0);
		ADC_para_temp.SNDR  -= buffer1[5]*10;
		buffer1[6] = (unsigned char) ADC_para_temp.SNDR;
		ADC_para_temp.SNDR  -= buffer1[6];
		ADC_para_temp.SNDR  *=100;
		buffer1[7] = '.';
		buffer1[8] = (unsigned char) ADC_para_temp.SNDR / 10;
		ADC_para_temp.SNDR  -= buffer1[8]*10;
		buffer1[9] = (unsigned char) ADC_para_temp.SNDR;
		buffer1[5] += '0';
		buffer1[6] += '0';
		buffer1[8] += '0';
		buffer1[9] += '0';

		buffer1[15] = (unsigned char) ADC_para_temp.SNR / 10;
		ADC_para_temp.SNR  -= buffer1[15]*10;
		buffer1[16] = (unsigned char) ADC_para_temp.SNR;
		ADC_para_temp.SNR  -= buffer1[16];
		ADC_para_temp.SNR  *=100;
		buffer1[17] = '.';
		buffer1[18] = (unsigned char) ADC_para_temp.SNR / 10;
		ADC_para_temp.SNR  -= buffer1[18]*10;
		buffer1[19] = (unsigned char) ADC_para_temp.SNR;
		buffer1[15] += '0';
		buffer1[16] += '0';
		buffer1[18] += '0';
		buffer1[19] += '0';
		
		
		buffer2[5] = (unsigned char) ADC_para_temp.ENOB / 10;
		ADC_para_temp.ENOB  -= buffer2[5]*10;
		buffer2[6] = (unsigned char) ADC_para_temp.ENOB;
		ADC_para_temp.ENOB  -= buffer2[6];
		ADC_para_temp.ENOB  *=100;
		buffer2[7] = '.';
		buffer2[8] = (unsigned char) ADC_para_temp.ENOB / 10;
		ADC_para_temp.ENOB  -= buffer2[8]*10;
		buffer2[9] = (unsigned char) ADC_para_temp.ENOB;
		buffer2[5] += '0';
		buffer2[6] += '0';
		buffer2[8] += '0';
		buffer2[9] += '0';

		buffer2[15] = '-';
		ADC_para_temp.THD *= -1.0;
		buffer2[16] = (unsigned char) ADC_para_temp.THD / 10;
		ADC_para_temp.THD  -= buffer2[16]*10;
		buffer2[17] = (unsigned char) ADC_para_temp.THD;
		ADC_para_temp.THD  -= buffer2[17];
		ADC_para_temp.THD  *=100;
		buffer2[18] = '.';
		buffer2[19] = (unsigned char) ADC_para_temp.THD / 10;
		ADC_para_temp.THD  -= buffer2[19]*10;
		buffer2[20] = (unsigned char) ADC_para_temp.THD;
		buffer2[16] += '0';
		buffer2[17] += '0';
		buffer2[19] += '0';
		
		Write_Disaply();
}


/*****************************************************************************************************************************************************************
*	Function name:  Display_ADC_para_Terminal_IO
*	Description	:	If the calculated ADC parameters are needed to display on Teminal IO of IAR workbench, 
*					then this function can be called when ADC parameters are calculated.
*	Arguments	:	None. The function uses "ADC_para" structure (which is globally defined for storing calculated ADC performance parameters) directly.
*   Return value:   None.
*   Note		: 	Currently this function is not used in the application code.  
******************************************************************************************************************************************************************/

void Display_ADC_para_Terminal_IO(void)
{
  printf("\nSNR:%f", ADC_para.SNR);
  printf("\nSNDR:%f", ADC_para.SNDR);
  printf("\nENOB:%f", ADC_para.ENOB);  
  printf("\nTHD:%f", ADC_para.THD);
  printf("\n========================================\n");
}


/*****************************************************************************************************************************************************************
*	Function name:  Init_TWI0
*	Description	:	This function initialises TWI peripheral of processor to which character display is connected on the ADSP-CM408 EZ-KIT.
*					The 50KHz baud rate is used and transmit request on single byte is configured.
*	Arguments	:	None. 
*   Return value:   None.
*   Note		: 	This function is called at the start only by Init_Display function.
******************************************************************************************************************************************************************/

void Init_TWI0(void)
{
    *pREG_TWI0_CTL 		= 0x00;  					// Reset the TWI controller
	*pREG_TWI0_MSTRCTL 	= 0;
	*pREG_TWI0_MSTRSTAT = ( ENUM_TWI_MSTRSTAT_BUFWRERR_YES | ENUM_TWI_MSTRSTAT_BUFRDERR_YES | ENUM_TWI_MSTRSTAT_DNAK_YES | 
						    ENUM_TWI_MSTRSTAT_ANAK_YES | ENUM_TWI_MSTRSTAT_LOSTARB_YES );
	*pREG_TWI0_ISTAT 	=  	0xFF;				//	Clearing all pending interrupt flags
    *pREG_TWI0_FIFOCTL 	= ( ENUM_TWI_FIFOCTL_RXFLUSH | ENUM_TWI_FIFOCTL_TXFLUSH );
	*pREG_TWI0_CTL		= 	BITM_TWI_CTL_EN | ((10 << BITP_TWI_CTL_PRESCALE) & BITM_TWI_CTL_PRESCALE);
	*pREG_TWI0_CLKDIV	= 	( ((0x64 << BITP_TWI_CLKDIV_CLKHI) & BITM_TWI_CLKDIV_CLKHI) |
							  ((0x64 << BITP_TWI_CLKDIV_CLKLO) & BITM_TWI_CLKDIV_CLKLO) );
	*pREG_TWI0_FIFOCTL	= 	ENUM_TWI_FIFOCTL_TXONEBYTE | ENUM_TWI_FIFOCTL_RXTWOBYTE;
}


/*****************************************************************************************************************************************************************
*	Function name:  Write_TWI_LCD_Command
*	Description	:	This is a internal function, not called by application directly. It sends command to the character display controller.
*					The command to be send is passed through function parameter.
*	Arguments	:	CHIP_ID	: 	This is the I2C Address of Character TWI display. This is passed as a parameter so that it should not break if additional TWI 
*								controlling	code is added e.g to control I2C swiches on EZ-KIT.
*					cmd		:	This is the command to be passed to character display controller. Refer datasheet of device for more details.
*					cmd_value:	some commands needs 1 byte parameter. That paramter should be passed through this field. 
*					cmd_size :	If the command has 1 byte parameter, this field must be passed as 3. If the command do not have parameter, this field should be 1.
*   Return value:   None.
*   Note		: 	For command to be send, first 0xFE byte needs to be send. This byte differentiates the command and data stream.
******************************************************************************************************************************************************************/

void Write_TWI_LCD_Command(unsigned char CHIP_ID, unsigned char cmd, unsigned char cmd_value, unsigned char cmd_size)
{  
	unsigned char cmd_buffer[3];
	cmd_buffer[0] = 0xFE;
	cmd_buffer[1] = cmd;
	cmd_buffer[2] = cmd_value;
	Write_TWI_LCD_Data(CHIP_ID, cmd_buffer, cmd_size);
}


/*****************************************************************************************************************************************************************
*	Function name:  Write_TWI_LCD_Data
*	Description	:	This is a internal function, not called by application directly. It sends character stream to be displayed on character display.
*					The character data stream to be send is passed through function parameter.
*	Arguments	:	CHIP_ID	: 	This is the I2C Address of Character TWI display. This is passed as a parameter so that it should not break if additional TWI controlling
*								code is added e.g to control I2C swiches on EZ-KIT.
*					*Tx_Buff:	This pointers points the character stream to be display
*					count	:	This fields instructs number of characters to be displayed/ 
*   Return value:   None.
*   Note		: 	polling based apporach is used for TWI peripheal to transmit the bytes.
******************************************************************************************************************************************************************/

void Write_TWI_LCD_Data(unsigned char CHIP_ID, unsigned char* Tx_Buff, unsigned char count)
{
  	*pREG_TWI0_MSTRADDR = CHIP_ID;
	
	unsigned char i;
	*pREG_TWI0_TXDATA8 = *Tx_Buff++;
	*pREG_TWI0_MSTRCTL = (count << 6) | BITM_TWI_MSTRCTL_EN;
	for(i=0; i<(count-1); i++)
	{
		while(!(*pREG_TWI0_ISTAT &	BITM_TWI_ISTAT_TXSERV));
		*pREG_TWI0_TXDATA8 = *Tx_Buff++;
		*pREG_TWI0_ISTAT |= 	BITM_TWI_ISTAT_TXSERV;
	}
		
    while(!(*pREG_TWI0_ISTAT & BITM_TWI_ISTAT_MCOMP));
	*pREG_TWI0_ISTAT |= 	BITM_TWI_ISTAT_TXSERV | BITM_TWI_ISTAT_MCOMP;
}
/******************************************************************************************************************************************************************/
