/*!
 *****************************************************************************
 * @file:    system.c
 * @brief:   System startup code for ADSP-CM40z
 * @version: $Revision: 18050 $
 * @date:    $Date: 2013-02-14 11:03:02 -0500 (Thu, 14 Feb 2013) $
 *-----------------------------------------------------------------------------
 *
Copyright (c) 2012 Analog Devices, Inc.

All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - Modified versions of the software must be conspicuously marked as such.
  - This software is licensed solely and exclusively for use with processors
    manufactured by or for Analog Devices, Inc.
  - This software may not be combined or merged with other code in any manner
    that would cause the software to become subject to terms and conditions
    which differ from those listed here.
  - Neither the name of Analog Devices, Inc. nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.
  - The use of this software may or may not infringe the patent rights of one
    or more patent holders.  This license does not release you from the
    requirement that you obtain separate licenses from these patent holders
    to use this software.

THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL
PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include <stdint.h>
#include "startup.h"
#include "system.h"

#ifndef DO_NOT_RELOCATE_IVT
extern void __relocated_vector_table;
#else
extern void __vector_table;
#endif

/*----------------------------------------------------------------------------
  External Clock Variables
 *---------------------------------------------------------------------------*/
volatile int _Processor_cycles_per_sec = -1;

/**
 * Initialize the system
 *
 * @param  none
 * @return none
 *
 * @brief  Setup the microcontroller system.
 *         Initialize the System and update the SystemFrequency variable.
 */
void SystemInit (void)
{
    /* Switch the Interrupt Vector Table Offset Register
     * (VTOR) to point to the relocated IVT in SRAM
     */

    __disable_irq();  // do all this in safe way

    // switch from boot ROM IVT to application's IVT
    // set the System Control Block, Vector Table Offset Register
#ifndef DO_NOT_RELOCATE_IVT
    SCB->VTOR = (uint32_t) &__relocated_vector_table;
#else
    SCB->VTOR = (uint32_t) &__vector_table;
#endif

	// set all three (USGFAULTENA, BUSFAULTENA, and MEMFAULTENA) fault enable bits
    // in the System Control Block, System Handler Control and State Register
    // otherwise these faults are handled as hard faults
	SCB->SHCSR = SCB_SHCSR_USGFAULTENA_Msk |
	             SCB_SHCSR_BUSFAULTENA_Msk |
	             SCB_SHCSR_MEMFAULTENA_Msk ;

#if (__FPU_PRESENT == 1) && (__FPU_USED == 1)
    // the FPU is disabled by default so  enable FPU (NEON and VFP)
    // set the System Control Block, Coprocessor Access Control Register bits:
    // CP10 = grant CP10 coprocessor privlidged and user mode access (full access)
    // CP11 = grant CP11 coprocessor privlidged and user mode access (full access)
    // (CP10 and CP11 MUST be the same or "BEHAIVOR IS UNPREDICTABLE")
	SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));  /* set CP10 and CP11 for Full Access */
#endif

	// flush instruction and data pipelines to insure assertion of new settings
	__ISB();  // MUST OCCURE IMMEDIATELY AFTER UPDATING SCB->CPACR!!!
	__DSB();
    __enable_irq();

}
